/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package applications;

import variants.am.*;
import factory.*;
import java.io.*;
import variants.am.gui.*;
import util.*;
import javax.swing.JPanel;
import javax.swing.JFrame;
import org.jdom.Document;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

public class Applications {

	public static void main(String[] args)
	{
		prueba(args);
	}
	
	public static void prueba(String[] args) {
		VariantFactory f = new AmVariantFactory();
		InputParser p=f.createInputParser("XML");
		
		try
		{
			FileInputStream s= new FileInputStream("/home/ignacio/sat.xml");
			Psystem ps=p.parse(s);
			OutputParser op=f.createOutputParser(ps, "XML");
			FileWriter fichero = new FileWriter("/home/ignacio/sat2.xml");
			PrintWriter pw = new PrintWriter(fichero);
			op.parse(pw);
			SystemPanel sp= new SystemPanel(new AmGuiModel((AmSimulator)f.createSimulator(ps, true, true)));
			JFrame jf = new JFrame();
			jf.setContentPane(sp);
			jf.setVisible(true);
			
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
		
		//compiler(args);

	}
	
	public static void simulator(String[] args){
		
		String inputFile = null, outputFile = null;
		if (args.length!=3)
		{
			printSimulatorHelp();
			return;
		}
		if (!args[1].toUpperCase().equals("-O"))
		{
			printSimulatorHelp();
			return;
		}
		inputFile=args[0];
		outputFile=args[2];
		
		VariantFactory f= new AmVariantFactory();
		InputParser parser=f.createInputParser("XML");
		try
		{
			Psystem ps = parser.parse(new FileInputStream(inputFile));
			
			AmSimulator sim =(AmSimulator) f.createSimulator(ps, true,true);
			FileOutputStream stream = new FileOutputStream(outputFile);
			sim.setInfoChannel(new PrintStream(stream));
			sim.setTime(true);
			while(sim.step());
			System.out.println("Environment: "+sim.getCurrentConfig().getEnvironment());
			System.out.println("Steps: "+sim.getCurrentConfig().getNumber());
			System.out.println("Time: "+sim.getTime()/1000+" s.");
			System.out.println("Halting configuration (No rule can be selected to be executed in the next step)");

			
		}
		catch(Exception ex)
		{
			System.out.println(ex.getMessage());
		}
		
		
	
	}

	/** The main method for the compiler program
	 * @param args
	 */
	public static void compiler(String[] args) {

		boolean verbose = false, help = false, xml = false;
		String inputFile = null, outputFile = null;
		int verbosity = 3;
		for (int i = 0; i < args.length; i++) {
			if (verbose) {
				try {
					verbosity = Integer.parseInt(args[i]);
				} catch (NumberFormatException ex) {
					help = true;
				}
			} else if (args[i].toUpperCase().equals("-V"))
				verbose = true;
			else if (args[i].toUpperCase().equals("-H"))
				help = true;
			else if (args[i].toUpperCase().equals("-XML"))
				xml = true;
			else if (inputFile == null)
				inputFile = args[i];
			else if (outputFile == null)
				outputFile = args[i];
			else {

				help = true;
			}

		}
		if (inputFile == null || help || !xml)
			printCompilerHelp();
		else {
			if (outputFile == null)
				outputFile = "output.xml";

			try {
				FileInputStream stream = new FileInputStream(inputFile);

				try {
					if (verbosity > 0)
						printCompilerInitMsg();
					VariantFactory factory = new AmVariantFactory();
					InputParser inputParser = factory
							.createInputParser("P-Lingua");
					inputParser.setVerbosityLevel(verbosity);
					Psystem ps = inputParser.parse(stream);
					if (ps != null) {
						inputParser.getInfoChannel().println(
								"Generating XML file...");
						try {
							Document d = ps.getDocument();
							XMLOutputter x = new XMLOutputter(Format
									.getPrettyFormat());
							FileWriter fichero = new FileWriter(outputFile);
							PrintWriter pw = new PrintWriter(fichero);
							x.output(d, pw);
						} catch (IOException ex) {
							inputParser.getErrorChannel().println(
									ex.getMessage());
						}
					}

				} catch (PlinguaCoreException ex) {
					System.out.println(ex.getMessage());
				}
			} catch (FileNotFoundException ex) {
				System.out.println(ex.getMessage());
			}
		}

	}
	

	/**
	 * A method to show the initial message for the command-line compiler.
	 */
	public static void printCompilerInitMsg() {
		System.out.println("P-LINGUA COMMAND-LINE COMPILER 1.0 beta");
		System.out
				.println("Copyright (C) 2008 Ignacio Perez-Hurtado (perezh@us.es)");
		System.out
				.println("                   Research Group On Natural Computing");
		System.out.println("                   http://www.gcn.us.es");
		System.out.println("This program comes with ABSOLUTELY NO WARRANTY.");
		System.out
				.println("This is free software, and you are welcome to redistribute it");
		System.out
				.println("under the conditions of the GNU General Public License version 3,");
		System.out
				.println("for details see the file gpl.txt or http://www.gnu.org/licenses/gpl.html");
		System.out.println("");

		System.out
				.println("For more information about P-Lingua see the software section of http://www.gcn.us.es/");
	}

	/**
	 * A method to show the help message for the command-line compiler
	 */
	public static void printCompilerHelp() {

		System.out.println("\nCommand line format:");
		System.out
				.println("plingua input_file -xml output_file [-v verbosity_level] [-h]");
		System.out.println("\nOptions:");
		System.out.println("input_file: The P-Lingua input file.");
		System.out.println("output_file: The XML output file.");
		System.out
				.println("-v verbosity_level: Verbosity level between 0 and 5");
		System.out.println("-h: Shows this help.");

	}
	
	public static void printSimulatorHelp() {
		System.out.println("\nCommand line format:");
		System.out
				.println("plingua_sim input_file.xml -o output_file");
		System.out.println("\nOptions:");
		System.out.println("input_file.xml: The XML input file.");
		System.out.println("output_file: The output file.");	}
	
	public static void printSimulatorInitMsg() {
		System.out.println("P-LINGUA COMMAND-LINE SIMULATOR 1.0 beta");
		System.out
				.println("Copyright (C) 2008 Ignacio Perez-Hurtado (perezh@us.es)");
		System.out
				.println("                   Research Group On Natural Computing");
		System.out.println("                   http://www.gcn.us.es");
		System.out.println("This program comes with ABSOLUTELY NO WARRANTY.");
		System.out
				.println("This is free software, and you are welcome to redistribute it");
		System.out
				.println("under the conditions of the GNU General Public License version 3,");
		System.out
				.println("for details see the file gpl.txt or http://www.gnu.org/licenses/gpl.html");
		System.out.println("");

		System.out
				.println("For more information about P-Lingua see the software section of http://www.gcn.us.es/");
	}

}
