/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */

package factory;

import java.util.Collection;
import util.Membrane;
import util.MultiSet;
import util.HashMultiSet;

/**
 * An abstract class for a Configuration
 * 
 * @author Ignacio Perez-Hurtado
 * 
 */
public abstract class Configuration {

	private Psystem psystem;

	private MultiSet<String> environment;

	private int number = 0;

	/**
	 * @param psystem
	 *            A P System
	 */
	public Configuration(Psystem psystem) {
		if (psystem == null)
			throw new NullPointerException("Null P system");
		this.psystem = psystem;
		environment = new HashMultiSet<String>();
	}

	/**
	 * @return Get the P System for this configuration
	 */
	public Psystem getPsystem() {
		return psystem;
	}

	/**
	 * @return Get the number of configuration, the initial one is 0
	 */
	public int getNumber() {
		return number;
	}

	/**
	 * Set the number of configuration
	 * 
	 * @param number
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/**
	 * @return A Multiset<String> for the environment
	 */
	public MultiSet<String> getEnvironment() {
		return environment;
	}

	/**
	 * @return A Collection<? extends Membrane> of membranes for this
	 *         configuration
	 */
	public abstract Collection<? extends Membrane> getMembranes();

	public abstract Configuration clone();

}
