/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) 2008 Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package factory;

import java.io.InputStream;
import java.io.StringReader;
import util.PlinguaCoreException;
import java.io.PrintStream;

/**
 * An abstract class for an input parser
 * 
 * @author Ignacio Perez-Hurtado
 * 
 */
public abstract class InputParser {

	private PrintStream errorChannel = System.out;

	private PrintStream warningChannel = System.out;

	private PrintStream infoChannel = System.out;

	private PrintStream msgChannel = System.out;

	private int verbosityLevel = 3;

	/**
	 * @return The PrintStream to write errors
	 */
	public PrintStream getErrorChannel() {
		return errorChannel;
	}

	/**
	 * Set the PrintStream to write errors
	 * 
	 * @param errorChannel
	 */
	public void setErrorChannel(PrintStream errorChannel) {
		this.errorChannel = errorChannel;
	}

	/**
	 * 
	 * @return The PrintStream to write information
	 */
	public PrintStream getInfoChannel() {
		return infoChannel;
	}

	/**
	 * Set the PrintStream to write information
	 * 
	 * @param infoChannel
	 */
	public void setInfoChannel(PrintStream infoChannel) {
		this.infoChannel = infoChannel;
	}

	/**
	 * @return The PrintStream to write warnings
	 */
	public PrintStream getWarningChannel() {
		return warningChannel;
	}

	/**
	 * Set the PrintStream to write warnings
	 * 
	 * @param warningChannel
	 */
	public void setWarningChannel(PrintStream warningChannel) {
		this.warningChannel = warningChannel;
	}

	/**
	 * @return the verbosity level (usually between 0-5)
	 */
	public int getVerbosityLevel() {
		return verbosityLevel;
	}

	/**
	 * Set the verbosity level (usually between 0-5)
	 * 
	 * @param verbosityLevel
	 */
	public void setVerbosityLevel(int verbosityLevel) {
		this.verbosityLevel = verbosityLevel;
	}

	/**
	 * @return The PrintStream to write messages
	 */
	public PrintStream getMsgChannel() {
		return msgChannel;
	}

	/**
	 * Set the PrintStream to write messages
	 * 
	 * @param msgChannel
	 */
	public void setMsgChannel(PrintStream msgChannel) {
		this.msgChannel = msgChannel;
	}

	/**
	 * Parse an InputStream and generate a new P system
	 * 
	 * @param stream
	 * @return a new P system or null if it has errors
	 * @throws PlinguaCoreException
	 */
	public abstract Psystem parse(InputStream stream)
			throws PlinguaCoreException;

	/**
	 * Parse a StringReader and generate a new P system
	 * 
	 * @param reader
	 * @return a new P system or null if it has errors
	 * @throws PlinguaCoreException
	 */
	public abstract Psystem parse(StringReader reader)
			throws PlinguaCoreException;

}
