/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) 2008 Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package factory;

import java.io.OutputStream;
import java.io.Writer;

/**
 * An abstract class for an output parser
 * 
 * @author Ignacio Perez-Hurtado
 * 
 */
public abstract class OutputParser {

	private Psystem psystem;

	/**
	 * @param psystem:
	 *            A P system
	 */
	public OutputParser(Psystem psystem) {
		this.psystem = psystem;
	}

	/**
	 * @return Get the P system of the output parser
	 */
	public Psystem getPsystem() {
		return psystem;
	}

	/**
	 * Parse the P system to an OutputStream
	 * 
	 * @param stream:
	 *            an OutputStream
	 * @return: True if it has success
	 */
	public abstract boolean parse(OutputStream stream);

	/**
	 * @param stream:
	 *            a Writer
	 * @return: True if it has success
	 */
	public abstract boolean parse(Writer stream);
}
