/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) 2008 Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package factory;

import util.Alphabet;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.jdom.Document;

import util.MultiSet;

import util.Membrane;
import util.RulesSet;

/**
 * An abstract class for a P System
 * 
 * @author Ignacio Perez-Hurtado
 * 
 */
public abstract class Psystem {

	private Alphabet alphabet;

	private RulesSet rules;

	private Map<String, MultiSet<String>> initialMultiSets;

	/**
	 * Create a new empty P system
	 */
	public Psystem() {
		alphabet = new Alphabet();
		rules = new RulesSet();
		initialMultiSets = new HashMap<String, MultiSet<String>>();
	}

	/**
	 * @return The alphabet of the P system
	 */
	public Alphabet getAlphabet() {
		return alphabet;
	}

	/**
	 * @return The rules set of the p syste,
	 */
	public RulesSet getRules() {
		return rules;
	}

	/**
	 * @return A Map<String,MultiSet<String>> label :: initial multiset
	 */
	public Map<String, MultiSet<String>> getInitialMultiSets() {
		return initialMultiSets;
	}

	/**
	 * @return A Collection of all membranes of the P system
	 */
	public abstract Collection<? extends Membrane> getMembranes();

	/**
	 * @return The first configuration of the P system
	 */
	public abstract Configuration getFirstConfiguration();

	/**
	 * @return An XML document that represents the P system
	 */
	public abstract Document getDocument();

}
