/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */

package util;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;

/**
 * A class for the Alphabet of a P system for P-Lingua objects. For instance: a,
 * b, c{0}, d{3,4}
 * 
 * @author Ignacio Perez-Hurtado
 * 
 */
public class Alphabet {

	private ArrayList alphabet;

	public Alphabet() {
		alphabet = new ArrayList(100);
	}

	/**
	 * Add a new object to the alphabet if it does not exist yet
	 * 
	 * @param object:
	 *            A String that represents a P-Lingua object
	 */
	public void addObject(String object) {
		AlphabetObject a = new AlphabetObject(object);
		int index = Collections.binarySearch(alphabet, a);
		if (index < 0)
			alphabet.add(-index - 1, a);

	}

	/**
	 * Return the name of an object without sub-indexes. For instance: 'a{0,1}'
	 * the name is 'a'
	 * 
	 * @param object:
	 *            A String that represents a P-Lingua object
	 * @return: The name of the object
	 */
	public static String getName(String object) {
		AlphabetObject a = new AlphabetObject(object);
		return a.getName();
	}

	/**
	 * Return an array of indexes of one object. For instance: 'a{4,3,7}' the
	 * array is [4,3,7]
	 * 
	 * @param object:
	 *            A String that represents a P-Lingua object
	 * @return an array of indexes
	 */
	public static int[] getIndexes(String object) {
		AlphabetObject a = new AlphabetObject(object);
		ArrayList indexes = a.getIndexes();
		int index[] = new int[indexes.size()];
		for (int i = 0; i < indexes.size(); i++)
			index[i] = ((Integer) indexes.get(i)).intValue();
		return index;
	}

	/**
	 * @return The size of the alphabet
	 */
	public int size() {
		return alphabet.size();
	}

	public Object[] toArray() {
		Object[] o = new Object[alphabet.size()];
		Iterator it = alphabet.iterator();
		int i = 0;
		while (it.hasNext()) {
			o[i] = it.next();
			i++;
		}
		return o;

	}

	public String toString() {
		return alphabet.toString();
	}

	static class AlphabetObject implements Comparable {
		private String name;

		private ArrayList indexes;

		private String object;

		public AlphabetObject(String object) {
			this.object = object;
			indexes = new ArrayList();
			int i = object.indexOf("{");
			if (i != -1) {
				int j = object.indexOf("}");
				if (j == object.length() - 1) {
					name = object.substring(0, i);
					String substring = object.substring(i + 1, j);
					String[] strIndexes = substring.split(",");
					for (int k = 0; k < strIndexes.length; k++)
						indexes
								.add(new Integer(Integer
										.parseInt(strIndexes[k])));
				} else
					name = object;
			} else
				name = object;
		}

		public String toString() {
			return object;
		}

		public String getName() {
			return name;
		}

		public ArrayList getIndexes() {
			return indexes;
		}

		public boolean equals(Object arg0) {
			return compareTo(arg0) == 0;
		}

		public int compareTo(Object arg0) {
			if (arg0 == null)
				return 1;
			if (this == arg0)
				return 0;
			if (getClass() != arg0.getClass())
				return object.compareTo(arg0.toString());
			final AlphabetObject other = (AlphabetObject) arg0;
			if (indexes.size() < other.indexes.size())
				return -1;
			if (indexes.size() > other.indexes.size())
				return 1;
			int cn = name.compareTo(other.name);
			if (cn != 0)
				return cn;
			int c = 0;
			int i = 0;
			while (i < indexes.size() && c == 0) {
				c = ((Integer) indexes.get(i))
						.compareTo((Integer) other.indexes.get(i));
				i++;
			}

			return c;
		}

	}

}