/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */

package util;

import org.jdom.Element;




/**
 * An abstract class for a Membrane
 * @author Ignacio Perez-Hurtado
 *
 */
/**
 * @author ignacio
 *
 */
/**
 * @author ignacio
 *
 */
public abstract class Membrane {
	
	private MultiSet<String> multiSet=null;
	
	private String label="";
	private int charge=0;

	
	public Membrane()
	{
		this("");
	}
	
	/**
	 * @param label
	 */
	public Membrane(String label)
	{
		if (label==null)
			throw new NullPointerException("Null label");
		multiSet=new HashMultiSet<String>();
		this.label=label;
	
	}
	
		
	/**
	 * @return The Multiset<String> of the membrane
	 */
	public MultiSet<String> getMultiSet()
	{
		return multiSet;
	}

	/**
	 * @return The numeric charge
	 */
	public int getCharge() {
		return charge;
	}

	/**
	 * Introduce the charge with a positive number for positive membranes,
	 * with a negative number for negative membranes and zero for neutral
	 * membranes or membranes wuth poralizationless
	 * @param charge
	 */
	public void setCharge(int charge) {
		this.charge=charge;
	}

	/**
	 * @return The label of the membrane
	 */
	public String getLabel() {
		return label;
	}

	
	/**
	 * Set the label of the membrane, this fuction throws a NullPointerException
	 * if the label is null
	 * @param label
	 */
	public void setLabel(String label) {
		if (label==null)
			throw new NullPointerException("Null label");
		this.label = label;
	}

	/**
	 * Return a String "+" for positive charges, a String "-" for negative
	 * charges and an empty String for neutral chages 
	 * @param charge
	 * @return A String
	 */
	public static String getChargeSymbol(int charge)
	{
		if (charge>0)
			return "+";
		else
		if (charge<0)
			return "-";
		else
			return "";
	}
	/**
	 * Similar to getChargeSymbol, but this returns a String "0" for
	 * neutral charges
	 * @param charge
	 * @return A String
	 */
	public static String getCompleteChargeSymbol(int charge)
	{
		if (charge>0)
			return "+";
		else
		if (charge<0)
			return "-";
		else
			return "0";
	}
		
	
	
	public abstract String toString();
	/**
	 * Disolve the membrane
	 * @throws SemanticsException
	 */
	public abstract void disolution() throws SemanticsException;
	/**
	 * Divide the membrane
	 * @return A new Membrane 
	 * @throws SemanticsException
	 */
	public abstract Membrane division() throws SemanticsException;
	public abstract Membrane clone();
	
	/**
	 * Return true if the membrane has been disolved
	 * @return A boolean
	 */
	public abstract boolean isDisolved();
	
	/**
	 * Return an Jdom Element that represents the membrane
	 * @return
	 */
	public abstract Element getElement();
	
	

}
