/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package util;

import java.util.Set;
import java.util.Collection;

/**
 * @author Ignacio Perez-Hurtado
 * 
 * An interface to define multisets
 * @param <E>
 *            The type of the objects in the multiset
 */
public interface MultiSet<E> extends Collection<E> {

	/**
	 * @param object
	 * @return The multiplicity of the object
	 */
	public int count(Object object);

	/**
	 * Count the number of subsets
	 * 
	 * @param objects
	 * @return The number of subsets
	 */
	public int countSubSets(Collection<?> objects);

	/**
	 * Add an object to the multiset
	 * 
	 * @param object
	 * @param multiplicity
	 * @return True if the multiset has been changed
	 */
	public boolean add(E object, int multiplicity);

	/**
	 * Add several objects to the multiset
	 * 
	 * @param objects
	 * @param multiplicity
	 * @return True if the multiset has been changed
	 */
	public boolean addAll(Collection<? extends E> objects, int multiplicity);

	/**
	 * Remove several objects of the multiset
	 * 
	 * @param object
	 * @param multiplicity
	 * @return True if the multiset has been changed
	 */
	public boolean remove(Object object, int multiplicity);

	/**
	 * Subtraction between multisets
	 * 
	 * @param objects
	 * @return True if the multiset has been changed
	 */
	public boolean subtraction(Collection<?> objects);

	/**
	 * Get the set of objects without repetitions Changes in this set will
	 * affect the multiset
	 * 
	 * @return A Set<E> of objects
	 */
	public Set<E> entrySet();

}
