/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package util;



import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Collection;
import java.util.Iterator;



import org.jdom.Element;

import java.util.NoSuchElementException;







public class RulesSet implements Collection<Rule>{
	
	private Map<String,Set<Rule>[]> rules=null;
	private int size=0;
	
	
	public RulesSet()
	{
		rules = new HashMap<String,Set<Rule>[]>();
	}
	
	public RulesSet(RulesSet anotherOne)
	{
		rules = new HashMap<String,Set<Rule>[]>(anotherOne.rules);
	}
	
		
	public boolean add(Rule rule)
	{
		String label = rule.getLabel();
		int index;
		if (rule.getCharge()<0)
			index=0;
		else
		if (rule.getCharge()==0)
			index=1;
		else
			index=2;
		
		Set<Rule> []list = rules.get(label);
		if (list == null)
		{
			Set <Rule>[] rulesByCharge = (Set <Rule>[])new Set[3];
			for(int i=0;i<3;i++)
				rulesByCharge[i]=new HashSet<Rule>();
			rules.put(label,rulesByCharge);
			list = rulesByCharge;
		}
		boolean b=list[index].add(rule);
		if (b) size++;
		return b;
		
		
	}
	
	
	
	public Collection<Rule>get(String label,int charge)
	{
		Set<Rule>[]set = rules.get(label);
		if (set==null)
			return null;
		int index;
		if (charge<0)
			index=0;
		else
		if (charge==0)
			index=1;
		else
			index=2;
		return set[index];
	}
	
	


	public String toString() {
		
		String str="";
		Iterator <String>it = rules.keySet().iterator();
		
		while(it.hasNext())
		{
			String label= it.next();
			Set<Rule> [] s = rules.get(label);
			for (int i=0;i<3;i++)
			{
				if (s[i].size()>0)
					str+="\nLabel: "+label+", Charge: "+Membrane.getChargeSymbol(i-1)+"\r\n";
				Iterator <Rule>it1=s[i].iterator();
				while(it1.hasNext())
				 str+=it1.next()+"\r\n";
			}
		}
		
		return str;
	}

	public boolean addAll(Collection<? extends Rule> arg0) {
		Iterator <? extends Rule>it=arg0.iterator();
		boolean b=false;
		while(it.hasNext())
			b = b || add(it.next());
		return b;
	}

	public void clear() {
		rules.clear();
		size=0;
	}

	public boolean contains(Object arg0) {
		if (!(arg0 instanceof Rule))
			return false;
		Rule r = (Rule)arg0;
		Set <Rule>[]s =rules.get(r.getLabel());
		if (s==null)
			return false;
		int index;
		if (r.getCharge()<0)
			index=0;
		else
		if (r.getCharge()==0)
			index=1;
		else
			index=2;
		return s[index].contains(arg0);
		
	}

	public boolean containsAll(Collection<?> arg0) {
		Iterator it=arg0.iterator();
		boolean b=true;
		while(it.hasNext() && b)
			b = b && contains(it.next()); 
		return b;
		
	}

	public boolean isEmpty() {
		return rules.isEmpty();
	}

	public Iterator<Rule> iterator() {
		return new RulesSetIterator();
	}

	public boolean remove(Object arg0) {
		if (!(arg0 instanceof Rule))
			return false;
		Rule r = (Rule)arg0;
		Set <Rule>[]s =rules.get(r.getLabel());
		if (s==null)
			return false;
		int index;
		if (r.getCharge()<0)
			index=0;
		else
		if (r.getCharge()==0)
			index=1;
		else
			index=2;
		boolean b= s[index].remove(arg0);
		if (b) size--;
		return b;
	}

	public boolean removeAll(Collection<?> arg0) {
		Iterator it=arg0.iterator();
		boolean b=false;
		while(it.hasNext())
			b = b || remove(it.next());
		return b;
	}

	public boolean retainAll(Collection<?> arg0) {
		Iterator <Rule>it = iterator();
		boolean b=false;
		while(it.hasNext())
		{
			Rule r = it.next();
			if (!(arg0.contains(r)))
			{
				it.remove();
				b=true;
			}
		}
		return b;
	}

	public int size() {
		return size;
	}

	public Object[] toArray() {
		Object o[] = new Object[size];
		Iterator <Rule>it = iterator();
		int i=0;
		while (it.hasNext())
		{
			o[i]=it.next();
			i++;
		}
		return o;
	}
	
	public Element getElement()
	{
		Element e = new Element("rules");
		
		Collection<String> c = rules.keySet();
		
		Iterator<String> it = c.iterator();
		
		while(it.hasNext())
		{
			String label= it.next();
			Set <Rule>[] s = rules.get(label);
			for (int i=0;i<3;i++)
			{
				Iterator<Rule> it1 = s[i].iterator();
				while(it1.hasNext())
				{
					Rule r=it1.next();
					e.addContent(r.getElement());
					
				}
			}
		}
		
		return e;
	}
	

	public <T> T[] toArray(T[] arg0) {
		T[] aux;
		if (arg0==null)
			throw new NullPointerException();
		if (arg0.length<size)
			aux = (T[])new Object[size];
		else
		if (arg0.length==size)
			aux=arg0;
		else
		{
			aux=arg0;
			aux[size]=null;
		}
		
		Iterator it = iterator();
		int i=0;
		while(it.hasNext())
		{
			
			Object obj = it.next();
			if (aux[i].getClass().isAssignableFrom(obj.getClass()))
				aux[i]=(T)obj;
			else
				throw new ArrayStoreException();
			i++;
		}
		return aux;
	}
	
	class RulesSetIterator implements Iterator<Rule>
	{
		private int index=0;
		private Iterator<String> it;
		private String currentLabel=null;
		private Iterator<Rule> it1=null;
		private int state=0;
		private int count=0;
		public RulesSetIterator()
		{
			it = rules.keySet().iterator();
		}
		public boolean hasNext() {
			return count<size;
		}
		
		
		public Rule next() {
			switch(state)
			{
			case 0:
				currentLabel=it.next();
				if (currentLabel==null)
					return null;
				state=1;
				return next();
				
			case 1:
				it1=rules.get(currentLabel)[index].iterator();
				state=2;
				return next();
				
			case 2:
				try
				{
				Rule r = it1.next();
				count++;
				return r;
				}
				catch(NoSuchElementException ex)
				{
					if (index<2)
					{
						index++;
						state=1;
						return next();
					}
					else
					{
						index=0;
						state=0;
						return next();
					}
				}
				
				
			default:
				return null;
			
				
			}
		}

		public void remove() {
			if (it1==null)
				throw new IllegalStateException();
			it1.remove();
		}
		
	}
	
	
	
	
	
	
	
	
	

}
/*
 * P-Lingua compiler 1.0 beta
 * 
 * Ignacio Perez-Hurtado
 * Research Group On Natural Computing
 * perezh@us.es
 * http://www.gcn.us.es
 * 
 * February, 2008
 */
