/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am;
import util.MultiSet;
import util.HashMultiSet;
import java.util.Collection;
import java.util.Iterator;

import util.Membrane;
import factory.Configuration;
import util.CellLikeMembrane;
import factory.Psystem;


import java.util.HashSet;
import java.util.HashMap;

import org.jdom.Attribute;
import org.jdom.Comment;
import org.jdom.Document;
import org.jdom.Element;

public class AmPsystem extends Psystem {

	
	private CellLikeMembrane skin=null;
	private HashMap<String,MultiSet<String>> inputMultiSets;
	
	public AmPsystem()
	{
		inputMultiSets=new HashMap<String,MultiSet<String>>();
	}
	
	
	
	public HashMap<String,MultiSet<String>> getInputMultiSet() {
		return inputMultiSets;
	}



	public CellLikeMembrane getSkinMembrane() {
		return skin;
	}
	
	
	
	public void setSkinMembrane(CellLikeMembrane skinMembrane) {
		this.skin = skinMembrane;
	}



	public Configuration getFirstConfiguration() {
		AmConfiguration config= new AmConfiguration(this);
		config.setNumber(0);
		config.setSkinMembrane(skin.clone());
		setInitialMultiSet(config.getSkinMembrane());
		return config;
	}
	
	private void setInitialMultiSet(CellLikeMembrane membrane)
	{
		MultiSet<String>m=getInitialMultiSets().get(membrane.getLabel());
		if (m!=null)
			membrane.getMultiSet().addAll(m);
		Iterator<CellLikeMembrane> it= membrane.childMembranesIterator();
		while(it.hasNext())
			setInitialMultiSet(it.next());
		
	}

	
	public Collection<? extends Membrane> getMembranes() {
		if (skin==null)
			return new HashSet<CellLikeMembrane>();
		else
			return skin.getAllMembranes();
	}
	
	public Document getDocument()
	{
		Document d = new Document();
		Element e = new Element("active_membrane_psystem");
		e.setAttribute(new Attribute("version","1.0"));
		Comment c = new Comment("File generated with P-Lingua Compiler 1.0 beta\n" +
				"See http://www.gcn.us.es for more information.");
		d.addContent(c);
		e.addContent(getInitConfigElement());
		e.addContent(getMultiSetsElement());
		e.addContent(getRulesElement());
		d.setRootElement(e);
		return d;
	}
	
		
	public Element getInitConfigElement()
	{
		Element e = new Element("init_config");
		if (skin!=null)
			e.addContent(skin.getElement());
		return e;
	}
	
	/**
	 * 
	 * @return An XML element that represents the rules set
	 */
	
	public Element getRulesElement()
	{
		return getRules().getElement();
	}
	/**
	 * 
	 * @return An XML element that represents the initial multisets
	 */
	
	public Element getMultiSetsElement()
	{
		Element e = new Element("multisets");
		Iterator <String>it = getInitialMultiSets().keySet().iterator();
		while(it.hasNext())
		{
			String label = it.next();
			Element e1= new Element("multiset");
			e1.setAttribute(new Attribute("label",label));
			MultiSet<String> m = getInitialMultiSets().get(label);
			
		
			Iterator<String> it1=m.entrySet().iterator();
		
			while(it1.hasNext())
			{
				
				String obj=it1.next();
				int mult=m.count(obj);
				Element e2= new Element("object");
				e2.setAttribute("name",obj);
				e2.setAttribute("multiplicity",Integer.toString(mult));
				e1.addContent(e2);
			}
			e.addContent(e1);
			

		}
		return e;
	}
	

}
