/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am;



import factory.InputParser;
import factory.OutputParser;
import factory.Psystem;
import factory.Simulator;
import factory.VariantFactory;

public class AmVariantFactory extends VariantFactory {

	@Override
	public InputParser createInputParser(String inputParserID) {
		
		if (inputParserID.equals("P-Lingua"))
			return new AmPLinguaInputParser();
		else
		if (inputParserID.equals("XML"))
			return new AmXMLInputParser();
		else
			return null;
		
	}

	@Override
	public OutputParser createOutputParser(Psystem psystem, String outputParserID) {
		if (outputParserID.equals("XML") && psystem instanceof AmPsystem)
			return new AmXMLOutputParser((AmPsystem)psystem);
		else
			return null;
	}

	@Override
	public Simulator createSimulator(Psystem psystem,boolean stepBackAllowed,boolean alternateStepsAllowed) {
		
		return new AmSimulator((AmPsystem)psystem);
		
	}

	
	@Override
	public Psystem createPsystem() {
		return new AmPsystem();
	}

	@Override
	public String getVariantName() {
		return "Active Membranes";
	}

}
