package variants.am;
import util.MultiSet;
import util.HashMultiSet;
import java.io.InputStream;
import java.io.StringReader;
import factory.Psystem;
import util.PlinguaCoreException;
import factory.InputParser;
import variants.am.AmPsystem;
import org.jdom.input.SAXBuilder;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import java.io.IOException;
import java.util.List;
import util.CellLikeMembrane;
import variants.am.rules.*;
public class AmXMLInputParser extends InputParser {
	
		
	private SAXBuilder builder=null;
	
	public AmXMLInputParser()
	{
		builder = new SAXBuilder();
	}
	
	private void throwException() throws PlinguaCoreException
	{
		throw new PlinguaCoreException("This XML file does not define a P system with active membranes.");
	}

	private CellLikeMembrane readMembranesRec(Element e,CellLikeMembrane parentMembrane) throws PlinguaCoreException
	{
		if (!e.getName().equals("membrane"))
			throwException();
		String label=e.getAttributeValue("label");
		String charge=e.getAttributeValue("charge");
		int ch=0;
		if (label==null)
			throwException();
		if (charge!=null)
		{
			try
			{
				ch=Integer.parseInt(charge);
				if (ch>0)
					ch=1;
				else
				if (ch<0)
					ch=-1;
			}
			catch(NumberFormatException ex)
			{
				if (charge.equals("+1"))
					ch=1;
				else
				if (charge.equals("-1"))
					ch=-1;
				else
				throwException();
			}
		}
		CellLikeMembrane m= new CellLikeMembrane(label,parentMembrane);
		m.setCharge(ch);
		List l = e.getChildren();
		for(int i=0;i<l.size();i++)
			readMembranesRec((Element)l.get(i),m);
		return m;	
	}
	
	private void readMembranes(Element e,AmPsystem ps) throws PlinguaCoreException
	{
		List l=e.getChildren();
		if (l==null)
			return;
		if (l.size()!=1)
			throwException();
		Element m=(Element)l.get(0);
		CellLikeMembrane skin=readMembranesRec(m,null);
		ps.setSkinMembrane(skin);
	}
	private void readMultiSets(Element e,AmPsystem ps) throws PlinguaCoreException
	{
		List l=e.getChildren();
		if (l==null)
			return;
		for (int i=0;i<l.size();i++)
		{
			Element m=(Element)l.get(i);
			if (!m.getName().equals("multiset"))
				throwException();
			String label=m.getAttributeValue("label");
			MultiSet<String> ms= new HashMultiSet<String>();
			List o=m.getChildren();
			for(int j=0;j<o.size();j++)
			{
				Element k=(Element)o.get(j);
				if (!k.getName().equals("object"))
					throwException();
				String name=k.getAttributeValue("name");
				String mul=k.getAttributeValue("multiplicity");
				if (name==null)
					throwException();
				int mulValue=1;
				if (mul!=null)
				{
					try
					{
						mulValue=Integer.parseInt(mul);
					}
					catch(NumberFormatException ex)
					{
						throwException();
					}
				}
				ms.add(name,mulValue);
			}
			ps.getInitialMultiSets().put(label, ms);
		}
	}
	
	private AmRule readSendRule(Element e,boolean inOut) throws PlinguaCoreException
	{
		AmRule r;
		String label=e.getAttributeValue("label");
		String charge=e.getAttributeValue("charge");
		
		int ch=0;
		if (label==null)
			throwException();
		if (charge!=null)
		{
			try
			{
				ch=Integer.parseInt(charge);
				if (ch>0)
					ch=1;
				else
				if (ch<0)
					ch=-1;
			}
			catch(NumberFormatException ex)
			{
				if (charge.equals("+1"))
					ch=1;
				else
				if (charge.equals("-1"))
					ch=-1;
				else 
					throwException();
			}
		}	
		List l = e.getChildren();
		String object=null;
		String object1=null;
		int newCharge=0;
		if (l==null)
			throwException();
		
		for(int i=0;i<l.size();i++)
		{
			Element o=(Element)l.get(i);
			String name=o.getAttributeValue("object");
			if (o.getName().equals("left_hand_rule"))
			{
				if (object!=null)
					throwException();
				object=name;
			}
			else
			if (o.getName().equals("right_hand_rule"))
			{
				if (object1!=null)
					throwException();
				object1=name;
				String newChargeStr=o.getAttributeValue("charge");
				
				if (newChargeStr!=null)
				{
				try
				{
					newCharge=Integer.parseInt(newChargeStr);
					if (newCharge<0)
						newCharge=-1;
					else
					if (newCharge>1)
						newCharge=1;
					else
						newCharge=0;
				
				}
				catch(NumberFormatException ex)
				{
					if (newChargeStr.equals("+1"))
						newCharge=1;
					else
					if (newChargeStr.equals("-1"))
						newCharge=-1;
					else
					if (newChargeStr.equals("0"))
						newCharge=0;
					else
					throwException();
				}
				}
			}
			else
				throwException();
			
		}
		if (object==null)
			throwException();
		if (object1==null)
			throwException();
		if (inOut)
			r=new AmSendInRule(label,ch,object,newCharge,object1);
		else
			r=new AmSendOutRule(label,ch,object,newCharge,object1);
		
		return r;
	}
	private AmRule readDissolutionRule(Element e) throws PlinguaCoreException
	{
		AmRule r;
		String label=e.getAttributeValue("label");
		String charge=e.getAttributeValue("charge");
		
		int ch=0;
		if (label==null)
			throwException();
		if (charge!=null)
		{
			try
			{
				ch=Integer.parseInt(charge);
				if (ch>0)
					ch=1;
				else
				if (ch<0)
					ch=-1;
			}
			catch(NumberFormatException ex)
			{
				if (charge.equals("+1"))
					ch=1;
				else
				if (charge.equals("-1"))
					ch=-1;
				else
				throwException();
			}
		}	
		List l = e.getChildren();
		String object=null;
		String object1=null;
		
		if (l==null)
			throwException();
		
		for(int i=0;i<l.size();i++)
		{
			Element o=(Element)l.get(i);
			String name=o.getAttributeValue("object");
			if (o.getName().equals("left_hand_rule"))
			{
				if (object!=null)
					throwException();
				object=name;
			}
			else
			if (o.getName().equals("right_hand_rule"))
			{
				if (object1!=null)
					throwException();
				object1=name;
				
				
			}
			else
				throwException();
			
		}
		if (object==null)
			throwException();
		if (object1==null)
			throwException();
		r=new AmDisolutionRule(label,ch,object,object1);
		return r;
	}
	private AmRule readDivisionRule(Element e) throws PlinguaCoreException
	{
		AmRule r;
		String label=e.getAttributeValue("label");
		String charge=e.getAttributeValue("charge");
		
		int ch=0;
		if (label==null)
			throwException();
		if (charge!=null)
		{
			try
			{
				ch=Integer.parseInt(charge);
				if (ch>0)
					ch=1;
				else
				if (ch<0)
					ch=-1;
			}
			catch(NumberFormatException ex)
			{
				if (charge.equals("+1"))
					ch=1;
				else
				if (charge.equals("-1"))
					ch=-1;
				else
				throwException();
			}
		}	
		List l = e.getChildren();
		String object=null;
		String newObject[]= new String[2];
		newObject[0]=null;
		newObject[1]=null;
		int newCharge[]= new int[2];
		newCharge[0]=0;
		newCharge[1]=0;
		int index=0;
		if (l==null)
			throwException();
		
		for(int i=0;i<l.size();i++)
		{
			Element o=(Element)l.get(i);
			String name=o.getAttributeValue("object");
			if (o.getName().equals("left_hand_rule"))
			{
				if (object!=null)
					throwException();
				object=name;
			}
			else
			if (o.getName().equals("right_hand_rule"))
			{
				if (index==2)
					throwException();
				
				newObject[index]=name;
				String newChargeStr=o.getAttributeValue("charge");
				
				if (newChargeStr!=null)
				{
				try
				{
					newCharge[index]=Integer.parseInt(newChargeStr);
					if (newCharge[index]<0)
						newCharge[index]=-1;
					else
					if (newCharge[index]>1)
						newCharge[index]=1;
				
				}
				catch(NumberFormatException ex)
				{
					if (newChargeStr.equals("+1"))
						newCharge[index]=1;
					else
					if (newChargeStr.equals("-1"))
						newCharge[index]=-1;
					else
					throwException();
				}
				}
				index++;
			}
			else
				throwException();
			
		}
		if (object==null)
			throwException();
		if (newObject[0]==null || newObject[1]==null)
			throwException();
		
		r=new AmDivisionRule(label,ch,object,newCharge[0],newObject[0],newCharge[1],newObject[1]);
		return r;
	}	
	
	
	private AmRule readEvolutionRule(Element e) throws PlinguaCoreException
	{
		AmRule r;
		String label=e.getAttributeValue("label");
		String charge=e.getAttributeValue("charge");
		
		int ch=0;
		if (label==null)
			throwException();
		if (charge!=null)
		{
			try
			{
				ch=Integer.parseInt(charge);
				if (ch>0)
					ch=1;
				else
				if (ch<0)
					ch=-1;
			}
			catch(NumberFormatException ex)
			{
				if (charge.equals("+1"))
					ch=1;
				else
				if (charge.equals("-1"))
					ch=-1;
				else
				throwException();
			}
		}	
		List l = e.getChildren();
		String object=null;
		MultiSet<String> ms= new HashMultiSet<String>();
		if (l==null)
			throwException();
		
		for(int i=0;i<l.size();i++)
		{
			Element o=(Element)l.get(i);
			String name=o.getAttributeValue("object");
			if (o.getName().equals("left_hand_rule"))
			{
				if (object!=null)
					throwException();
				object=name;
			}
			else
			if (o.getName().equals("right_hand_rule"))
			{
				String multiplicity=o.getAttributeValue("multiplicity");
				int mul=1;
				if (multiplicity!=null)
				{
				try
				{
					mul=Integer.parseInt(multiplicity);
					if (mul<1)
						throwException();
					
				}
				catch(NumberFormatException ex)
				{
					throwException();
				}
				}
				ms.add(name,mul);
			}
			else
				throwException();
			
		}
		if (object==null)
			throwException();
		
		r=new AmEvolutionRule(label,ch,object,ms);
		return r;
	}
	
	
	private void readRules(Element e,AmPsystem ps) throws PlinguaCoreException
	{
		
		List l = e.getChildren();
		if (l==null)
			return;
		for (int i=0;i<l.size();i++)
		{
			Element r = (Element)l.get(i);
			
			AmRule rule=null;
			if (r.getName().equals("evolution_rule"))
			{
				rule=readEvolutionRule(r);
			}
			else
			if (r.getName().equals("send_in_rule"))
			{
				rule=readSendRule(r,true);
			}
			else
			if (r.getName().equals("send_out_rule"))
			{
				rule=readSendRule(r,false);
			}
			else
			if (r.getName().equals("division_rule"))
			{
				rule=readDivisionRule(r);
			}
			else
			if (r.getName().equals("dissolution_rule"))
			{
				rule=readDissolutionRule(r);
			}
			else
				throwException();
					
			ps.getRules().add(rule);
					
		}
	}
	
	private Psystem readDocument(Document d) throws PlinguaCoreException
	{
		AmPsystem psystem= new AmPsystem();
		Element root=null,membranes=null,multisets=null,rules=null;
		root=d.getRootElement();
		if (!root.getName().equals("active_membrane_psystem"))
			throwException();
		List l = root.getChildren();
		for (int i=0;i<l.size();i++)
		{
			Element e = (Element)l.get(i);
			if (e.getName().equals("init_config"))
			{
				if (membranes!=null)
					throwException();
				membranes=e;
				readMembranes(e,psystem);
			}
			else
			if (e.getName().equals("multisets"))
			{
				if (multisets!=null)
					throwException();
				multisets=e;
				readMultiSets(e,psystem);
			}
			else
			if (e.getName().equals("rules"))
			{
				if (rules!=null)
					throwException();
				rules=e;
				readRules(e,psystem);
			}
			else
			throwException();
		}
		if (membranes==null)
			throwException();
		return psystem;
	}
	
	public Psystem parse(InputStream stream) throws PlinguaCoreException {
		
		try
		{
		try
		{
			Document d=builder.build(stream);
			return readDocument(d);
		}
		catch(JDOMException ex)
		{
			throw new PlinguaCoreException(ex.getMessage());
		}
		}
		catch(IOException ex)
		{
			throw new PlinguaCoreException(ex.getMessage());
		}
		
		
	}

	@Override
	public Psystem parse(StringReader reader) throws PlinguaCoreException {
		try
		{
		try
		{
			Document d=builder.build(reader);
			return readDocument(d);
		}
		catch(JDOMException ex)
		{
			throw new PlinguaCoreException(ex.getMessage());
		}
		}
		catch(IOException ex)
		{
			throw new PlinguaCoreException(ex.getMessage());
		}
	}

}
