/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.gui;
import util.HashMultiSet;
import java.awt.Rectangle;
import javax.swing.JPanel;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.Color;
import java.awt.BasicStroke;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Dimension;
import util.CellLikeMembrane;
import util.Membrane;
import java.util.Iterator;
import util.MultiSet;
import java.util.Iterator;
import util.Alphabet;
public class CellLikeMembranePanel extends JPanel
{
	public static final long serialVersionUID=1;
	private CellLikeMembrane membrane;
	private CellLikeMembrane markedMembrane;
	
	private Color colorEntorno;
	private Color internalMembraneColor;
	private Color membraneColor;
	private Color markedMembraneColor;
	private Color newObjectColor;
	private Color inputObjectColor;
	private static final String fontName="Helvetica";

	
	private final static int margenPorcentaje = 15;
	private final static BasicStroke stroke = new BasicStroke(2.0f);
	private final static Font font = new Font(fontName,Font.PLAIN,12);

	
	CellLikeMembranePanel()
	{
		super();
		
		membrane=null;
		colorEntorno = Color.white;
		internalMembraneColor=Color.white;
		membraneColor = Color.black;
		inputObjectColor=Color.blue;
		markedMembraneColor = Color.cyan;
		newObjectColor=Color.red;
		
		
	}
	
	

	public void paintComponent(Graphics g)
	{
		
		clear(g);
		Dimension d;
		Graphics2D g2d = (Graphics2D)g;
		d = getSize();
		
		pintaMembrana(g2d,membrane,d.width/20,d.width/20,d.width-d.width/20,d.height-d.width/20);
	}
	
	private static final void pintaEtiqueta(Graphics2D g2d, String texto,int x0,int y0,int x1,int y1)
	{
		double l = 1.5*(x1-x0)/margenPorcentaje;
		
		g2d.setFont(font);
	
		Rectangle2D r=g2d.getFontMetrics(font).getStringBounds(texto,g2d);
		
		double h = r.getHeight();
		double p=Math.round(l*20/h); /* antes 12 */
		g2d.setFont( new Font(fontName,Font.PLAIN,(int)p));
		
		r=g2d.getFontMetrics().getStringBounds(texto,g2d);
	
		g2d.drawString(texto, x1+1, y1); 
	}
	

	
	private  final void pintaMultiSet(Graphics2D g2d,MultiSet<String> ms[],int x0,int y0,int x1,int y1)
	{
		
		int lines=0;
		int hmax=60;
		int h;
		do
		{
			lines++;
			h=calcSizeMultiSet(g2d,ms,x0,y0,x1,y1,lines);
		}while((h==-1 || h>hmax)&& lines<17);
		if (h!=-1 && h<=hmax)
		{
			
			Rectangle2D r;
			Iterator<String>it ;
			Alphabet a = new Alphabet();
			boolean end=false,neg=false;
			int x=x0,y=y0;
			int height=(y1-y0)/lines;
			if (height<=5)
				end=true;
		
			for(int i=0;i<ms.length && !end;i++)
			{
				if (i==0)
					g2d.setPaint(membraneColor);
				else
					g2d.setPaint(newObjectColor);
				it= ms[i].entrySet().iterator();
			
				while(it.hasNext() && !end)
				{
					String o = it.next();
					if (o.charAt(0)=='n')
					{
						o=o.substring(1);
						neg=true;
					}
					else
						neg=false;
					r=pintaObjeto(g2d,a.getName(o),a.getIndexes(o),ms[i].count(o),x,y,height,neg,false);
				    if (x+r.getWidth()>x1)
				    {
				    	x=x0;
				    	y+=height;
				    	if (y+height>y1)
				    		end=true;
			        }
				    if (!end)
				    {
				    	
				   
				    	pintaObjeto(g2d,a.getName(o),a.getIndexes(o),ms[i].count(o),x,y,height,neg,true);
				       	x+=r.getWidth()+ (x1-x0)/20;
				    }
			   	}
			}
			
		}
	}
	
	private static final int calcSizeMultiSet(Graphics2D g2d,MultiSet<String> []ms,int x0,int y0,int x1,int y1,int lines)
	{
		Rectangle2D r;
		int x=x0,y=y0;
		Iterator<String>it ;
		Alphabet a = new Alphabet();
		boolean end=false,neg=false;
		int height=(y1-y0)/lines;
		if (height<=5)
			end=true;
	
		for(int i=0;i<ms.length && !end;i++)
		{
			it= ms[i].entrySet().iterator();
			
			while(it.hasNext() && !end)
			{
				String o = it.next();
				if (o.charAt(0)=='n')
				{
					o=o.substring(1);
					neg=true;
				}
				else
					neg=false;
				r=pintaObjeto(g2d,a.getName(o),a.getIndexes(o),ms[i].count(o),x,y,height,neg,false);
			    if (x+r.getWidth()>x1)
			    {
			    	x=x0;
			    	y+=height;
			    	if (y+height>y1)
			    		end=true;
		        }
			    if (!end)
		   	  		x+=r.getWidth()+ (x1-x0)/20;
		   	}
		}
		if (end)
			return -1;
		else
			return height;
	}
	
	private static final void pintaCarga(Graphics2D g2d, String texto,int x0,int y0,int x1,int y1)
	{
		
		double l = 1.5*(x1-x0)/margenPorcentaje;
		g2d.setFont(font);
		Rectangle2D r=g2d.getFontMetrics(font).getStringBounds(texto,g2d);
		double h = r.getHeight();
		double p=Math.round(l*20/h);
		g2d.setFont( new Font(fontName,Font.PLAIN,(int)p));
		r=g2d.getFontMetrics().getStringBounds(texto,g2d);
		h=Math.round(r.getHeight()/2);
		g2d.drawString(texto, x1+1, y0+(int)h); 
	}
	
	
	
	private static final Rectangle2D pintaObjeto(Graphics2D g2d,String object,int []subIndexes,int multiplicity,int x0, int y0, int height,boolean neg,boolean paint)
	{
		int width;
		int width2=0;
		Rectangle2D r1;
		String subIndexStr=null;
		if (subIndexes!=null)
		{
			subIndexStr="";
			for (int i=0;i<subIndexes.length;i++)
			{
				subIndexStr+=subIndexes[i];
				if (i<subIndexes.length-1)
					subIndexStr+=",";
			}
		}
		g2d.setFont(font);
		Rectangle2D r=g2d.getFontMetrics().getStringBounds(object,g2d);
		double h = r.getHeight();
		double p=Math.round((height*12)/h);
		g2d.setFont( new Font(fontName,Font.PLAIN,(int)p));
		r=g2d.getFontMetrics().getStringBounds(object,g2d);
		width=(int)Math.round(r.getWidth());
		height=(int)Math.round(r.getHeight());
		int ascent=g2d.getFontMetrics().getAscent();
		int descent=g2d.getFontMetrics().getDescent();
		int leading=g2d.getFontMetrics().getLeading();
	    if (paint)
	    	g2d.drawString(object, x0, y0+leading+ascent);
		if (paint && neg)
		{
			g2d.setStroke(stroke);	
		
			g2d.drawLine(x0+2,y0+leading+descent,x0+width-2,y0+leading+descent);
		}
		
	
		if (subIndexStr!=null)
		{
			g2d.setFont( new Font(fontName,Font.PLAIN,(int)p/3));
			if (paint)
				g2d.drawString(subIndexStr, x0+width, y0+leading+ascent); 
			r1=g2d.getFontMetrics().getStringBounds(subIndexStr,g2d);
		    if (r1.getWidth()>width2)
		    	width2=(int)Math.round(r1.getWidth());
		}
		if (multiplicity>1)
		{
			g2d.setFont( new Font(fontName,Font.PLAIN,(int)p/3));
			if (paint)
				g2d.drawString(Integer.toString(multiplicity), x0+width, (int)Math.round(y0+leading+1.75*descent));
			r1=g2d.getFontMetrics().getStringBounds(Integer.toString(multiplicity),g2d);
			if (r1.getWidth()>width2)
				width2=(int)Math.round(r1.getWidth());
		}
		r = new Rectangle(width+width2,height);
	
		return r;
		
		
	}
	
	
	
	public static final void pintaTexto(Graphics2D g2d,String texto,int x0,int y0,int x1,int y1)
	{
		
		g2d.setFont(font);
		Rectangle2D r=g2d.getFontMetrics().getStringBounds(texto,g2d);
		double h = r.getHeight();
		double p=Math.round(((y1-y0)*10)/h);
		
		g2d.setFont( new Font(fontName,Font.PLAIN,(int)p));
		r=g2d.getFontMetrics().getStringBounds(texto,g2d);
		h = Math.round(r.getHeight());
		g2d.drawString(texto, (int)x0+2, y0+(int)h); 
	
		
	}
	
	private void pintaMembrana(Graphics2D g2d,CellLikeMembrane membrane,int x0,int y0,int x1,int y1)
	{
		
		
		g2d.getFontMetrics(font);
		
		x1 = x1 - (x1-x0)/margenPorcentaje;
		
		if (membrane!=null && (x1-x0)>0 && (y1-y0)>0)
		{
			
			g2d.setFont(font);
			
			RoundRectangle2D.Double r;
			
			
			r=new RoundRectangle2D.Double(x0,y0,x1-x0,y1-y0,20,20);
			g2d.setPaint(internalMembraneColor);
			g2d.fill(r);
			
			if (membrane==markedMembrane)
				g2d.setPaint(markedMembraneColor);
			else
				g2d.setPaint(membraneColor);
			
			if ((((x1-x0)*(y1-y0)*16)/1000000)<2)
			{
				g2d.setStroke(stroke);	
			}
			else
			{
				g2d.setStroke(new BasicStroke(((x1-x0)*(y1-y0)*16)/1000000));
			}
		
			g2d.draw(r);
			
			pintaEtiqueta(g2d,membrane.getLabel(),x0,y0,x1,y1);
			pintaCarga(g2d,Membrane.getCompleteChargeSymbol(membrane.getCharge()),x0,y0,x1,y1);
			MultiSet<String> ms[] = new MultiSet[2];
			ms[0] = new HashMultiSet<String>(membrane.getMultiSet());
			ms[1] = membrane.getNewMultiSet();
			
			ms[0].subtraction(ms[1]);
			
		
			if (membrane.childMembranesCount()==0)
				pintaMultiSet(g2d,ms,x0,y0,x1,y1);
			
			if (membrane.childMembranesCount()!=0)
			{
				int nbox;
				if (membrane.getMultiSet().size()==0)
					nbox=membrane.childMembranesCount();
				else
					nbox=membrane.childMembranesCount()+1;
				int raiz = (int)Math.floor(Math.sqrt(nbox));
				int m,n;
				if (raiz*raiz == nbox)
				{
					m=raiz;
					n=raiz;
				}
				else if ((raiz*(raiz+1))>=nbox)
				{
					if ((x1-x0)>= (y1-y0))
					{
						m=raiz+1;
						n=raiz;
					}
					else
					{
						n=raiz+1;
						m=raiz;
					}
					
				}
				else
				{
					m=raiz+1;
					n=raiz+1;
				}
				
				int i,j,a,b,a1,b1,mx0,my0,mx1,my1;
				i=0;
				j=0;
				
				Iterator <CellLikeMembrane>it = membrane.childMembranesIterator();
				while(it.hasNext())
				{
					
					a=x0+i*((x1-x0)/m);
					b=y0+j*((y1-y0)/n);
					a1=a+(x1-x0)/m;
					b1=b+(y1-y0)/n;
					mx0=((x1-x0)/m)/margenPorcentaje;
					mx1=((x1-x0)/m)/margenPorcentaje;
					my0=((x1-x0)/m)/margenPorcentaje;
					my1=((x1-x0)/m)/margenPorcentaje;
					if (i>0 && i<m)
						mx0=mx0/2;
					if (j>0 && j<n)
						my0=my0/2;
					try
					{
						
						pintaMembrana(g2d,it.next(),a+mx0,b+my0,a1-mx1,b1-my1);
					}catch(Exception e){}
					i++;
					if (i==m)
					{
						i=0;
						j++;
					}
				}
				if (membrane.getMultiSet().size()>0)
				{
					a=x0+i*((x1-x0)/m);
					b=y0+j*((y1-y0)/n);
					a1=a+(x1-x0)/m;
					b1=b+(y1-y0)/n;
					mx0=((x1-x0)/m)/margenPorcentaje;
					mx1=((x1-x0)/m)/margenPorcentaje;
					my0=((x1-x0)/m)/margenPorcentaje;
					my1=((x1-x0)/m)/margenPorcentaje;
					if (i>0 && i<m)
						mx0=mx0/2;
					if (j>0 && j<n)
						my0=my0/2;
					
				
					pintaMultiSet(g2d,ms,a+mx0,b+my0,a1-mx1,b1-my1);
				}
				
					
			}
			
			
			
		}
	}
	

	protected void clear(Graphics g)
	{
		super.paintComponent(g);
	}

	public Membrane getMembrana()
	{
		return membrane;
	}

	public void setMembrane(CellLikeMembrane membrane)
	{
		this.membrane = membrane;
	}
	
	
	
	
	public Color getColorEntorno() {
		return colorEntorno;
	}

	public void setColorEntorno(Color colorEntorno) {
		this.colorEntorno = colorEntorno;
	}



	public Color getInternalMembraneColor() {
		return internalMembraneColor;
	}



	public void setInternalMembraneColor(Color internalMembraneColor) {
		this.internalMembraneColor = internalMembraneColor;
	}



	public Membrane getMarkedMembrane() {
		return markedMembrane;
	}



	public void setMarkedMembrane(CellLikeMembrane markedMembrane) {
		this.markedMembrane = markedMembrane;
	}



	public Color getMarkedMembraneColor() {
		return markedMembraneColor;
	}



	public void setMarkedMembraneColor(Color markedMembraneColor) {
		this.markedMembraneColor = markedMembraneColor;
	}



	public Color getMembraneColor() {
		return membraneColor;
	}



	public void setMembraneColor(Color membraneColor) {
		this.membraneColor = membraneColor;
	}



	public Membrane getMembrane() {
		return membrane;
	}
	
	
	
	
	
	
	
	
	
	
	
}
