/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.gui;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel; 
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JToolBar;
import javax.swing.border.TitledBorder;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintStream;
import java.io.ByteArrayOutputStream;
import java.util.Locale;


public class ConsolePanel extends JPanel implements AmGuiView, ActionListener {

	private static final long serialVersionUID = 1L;
	private JTextArea jTextArea=null;
	private JScrollPane jScrollPane = null;
	private JToolBar jToolBar = null;
	private Component owner=null;
	private JButton jButton = null;
	private JButton jButton1 = null;
	private PrintStream printStream=null;
	
	public ConsolePanel(Component owner)
	{
		this.owner=owner;
		initialize();
		printStream=new MyPrintStream();
	}
	
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setLayout(new BorderLayout());
		
		
		this.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.gray, 1),
				"Console",
				TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION,
				null,
				null));
		this.add(getJScrollPane(), BorderLayout.CENTER);
		this.add(getJToolBar(), BorderLayout.SOUTH);
		updateView();
	}
	/**
	 * This method initializes jToolBar	
	 * 	
	 * @return javax.swing.JToolBar	
	 */
	private JToolBar getJToolBar() {
		if (jToolBar == null) {
			jToolBar = new JToolBar();
			jToolBar.setFloatable(false);
			jToolBar.add(getJButton());
			jToolBar.add(getJButton1());
		}
		return jToolBar;
	}
	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText("Save");
			jButton.addActionListener(this);
		}
		return jButton;
	}
	/**
	 * This method initializes jButton1
	 * 
	 * @return javax.swing.JButton1
	 */
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setText("Clear");
			jButton1.addActionListener(this);
		}
		return jButton1;
	}
	
	/**
	 * This method initializes jScrollPane	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
			jScrollPane.setViewportView(getJTextArea());
		}
		return jScrollPane;
	}
	
	private JTextArea getJTextArea(){
		if (jTextArea==null)
		{
			jTextArea=new JTextArea();
			jTextArea.setEditable(false);
		}
		return jTextArea;
	}
	
	public void updateView() {
		// TODO Auto-generated method stub
		
	}

	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub
		
	}
	
	class MyPrintStream extends PrintStream
	{
		public MyPrintStream()
		{
			super(new ByteArrayOutputStream());
		}

	
		@Override
		public void print(boolean arg0) {
			// TODO Auto-generated method stub
			jTextArea.append(Boolean.toString(arg0));
		
		}

		@Override
		public void print(char arg0) {
			// TODO Auto-generated method stub
			jTextArea.append(Character.toString(arg0));
			
		}

		@Override
		public void print(char[] arg0) {
			// TODO Auto-generated method stub
			jTextArea.append(new String(arg0));
			
		}

		@Override
		public void print(double arg0) {
			jTextArea.append(Double.toString(arg0));
			
		}

		@Override
		public void print(float arg0) {
			// TODO Auto-generated method stub
			jTextArea.append(Float.toString(arg0));
		
		}

		@Override
		public void print(int arg0) {
			jTextArea.append(Integer.toString(arg0));
			
		}

		@Override
		public void print(long arg0) {
			jTextArea.append(Long.toString(arg0));
		
		}

		@Override
		public void print(Object arg0) {
			jTextArea.append(arg0.toString());
			
		}

		@Override
		public void print(String arg0) {
			// TODO Auto-generated method stub
			jTextArea.append(arg0);
			
		}

	

		@Override
		public void println() {
			// TODO Auto-generated method stub
			jTextArea.append("\n");
		
		}

		@Override
		public void println(boolean arg0) {
			jTextArea.append(Boolean.toString(arg0)+"\t\n");
			
		}

		@Override
		public void println(char arg0) {
			// TODO Auto-generated method stub
			jTextArea.append(Character.toString(arg0)+'\n');
			
		}

		@Override
		public void println(char[] arg0) {
			jTextArea.append(new String(arg0)+'\n');
			
		}

		@Override
		public void println(double arg0) {
			jTextArea.append(Double.toString(arg0)+'\n');
			
		}

		@Override
		public void println(float arg0) {
			jTextArea.append(Float.toString(arg0)+'\n');
		
		}

		@Override
		public void println(int arg0) {
			jTextArea.append(Integer.toString(arg0)+'\n');
		
		}

		@Override
		public void println(long arg0) {
			jTextArea.append(Long.toString(arg0)+'\n');
		
		}

		@Override
		public void println(Object arg0) {
			jTextArea.append(arg0.toString()+'\n');
		
		}

		@Override
		public void println(String arg0) {
			
			jTextArea.append(new String(arg0)+"\n");
			
		}

	
		
	}

	public PrintStream getPrintStream() {
		return printStream;
	}
	

}
