/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.gui;

import javax.swing.JPanel;
import javax.swing.JSplitPane;
import java.awt.BorderLayout;
import javax.swing.BorderFactory;
import java.awt.Color;
import javax.swing.border.TitledBorder;
import javax.swing.JDialog;
import javax.swing.JToolBar;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.JButton;

import java.awt.Dimension;

import java.awt.Graphics;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import util.CellLikeMembrane;
import variants.am.AmSimulator;
import variants.am.AmConfiguration;
import java.awt.Component;

public class MembranesPanel extends JPanel implements ActionListener,TreeSelectionListener, AmGuiView {

	private static final long serialVersionUID = 1L;

	private JSplitPane jSplitPane = null;

	private JToolBar jToolBar = null;

	private JScrollPane jScrollPane = null;

	private CellLikeMembranePanel panelMembrana=null;

	private JButton jButton = null;

	private JButton jButton1 = null;

	private JButton jButton2 = null;

	private JButton jButton3 = null;
	
	private JTree jTree = null;

	private AmSimulator simulator = null;

	private MembraneTreeModel membraneTreeModel = null; // @jve:decl-index=0:
	
	private AmGuiModel model;
	
	private Component owner;
	
	private AlphabetPanel objectsPanel=null;

	/**
	 * This is the default constructor
	 */
	public MembranesPanel(AmGuiModel model,Component owner,AlphabetPanel objectsPanel) {
		super();
		this.model=model;
		this.simulator = model.getSimulator();
		
		this.owner=owner;
		this.objectsPanel=objectsPanel;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {

		this.setSize(300, 200);
		this.setLayout(new BorderLayout());
				
		this.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.gray, 1), "Membrane Structure",
				TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION,
				null,
				null));
		
		
		this.add(getJSplitPane(), BorderLayout.CENTER);
		this.add(getJToolBar(), BorderLayout.SOUTH);
		updateView();
		updateButtons();
	}

	/**
	 * This method initializes jSplitPane
	 * 
	 * @return javax.swing.JSplitPane
	 */
	private JSplitPane getJSplitPane() {
		if (jSplitPane == null) {
			jSplitPane = new JSplitPane();
			jSplitPane.setLeftComponent(getJScrollPane());
			jSplitPane.setRightComponent(getPanelMembrana());
		}
		return jSplitPane;
	}

	/**
	 * This method initializes jToolBar
	 * 
	 * @return javax.swing.JToolBar
	 */
	private JToolBar getJToolBar() {
		if (jToolBar == null) {
			jToolBar = new JToolBar();
			jToolBar.setFloatable(false);
			jToolBar.add(getJButton());
			jToolBar.add(getJButton1());
			jToolBar.add(getJButton2());
			jToolBar.add(getJButton3());
		}
		return jToolBar;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane
					.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
			jScrollPane.setViewportView(getJTree());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jDesktopPane
	 * 
	 * @return javax.swing.JDesktopPane
	 */
	private CellLikeMembranePanel getPanelMembrana() {
		if (panelMembrana == null) {
			panelMembrana = new CellLikeMembranePanel();
			panelMembrana.setBorder(BorderFactory
					.createLineBorder(Color.gray, 1));
		}
		return panelMembrana;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButton() {
		if (jButton == null) {
			jButton = new JButton();
			jButton.setText("Reset");
			jButton.addActionListener(this);
		}
		return jButton;
	}

	/**
	 * This method initializes jButton1
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButton1() {
		if (jButton1 == null) {
			jButton1 = new JButton();
			jButton1.setText("Step Back");
			jButton1.addActionListener(this);
		}
		return jButton1;
	}

	/**
	 * This method initializes jButton2
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButton2() {
		if (jButton2 == null) {
			jButton2 = new JButton();
			jButton2.setText("Step Forward");
			jButton2.addActionListener(this);
		}
		return jButton2;
	}
	
	private JButton getJButton3() {
		if (jButton3 == null) {
			jButton3 = new JButton();
			jButton3.setText("Run...");
			jButton3.addActionListener(this);
		}
		return jButton3;
	}

	/**
	 * This method initializes jTree
	 * 
	 * @return javax.swing.JTree
	 */
	private JTree getJTree() {
		if (jTree == null) {
			jTree = new JTree(new MembraneTreeModel());
			jTree.addTreeSelectionListener(this);
		}
		return jTree;
	}

	public void updateView() {
		
		AmConfiguration config=(AmConfiguration)simulator.getCurrentConfig();
		if (config.getSkinMembrane() == null) {
			jTree.setModel(new MembraneTreeModel());
			membraneTreeModel = null;
			updateMembranePanel(null,null,"Membrane Structure",true);
			

		} else {
			
				membraneTreeModel = new MembraneTreeModel();
				membraneTreeModel.updateTree(config.getSkinMembrane());
				jTree.setModel(membraneTreeModel);
			
			
			jTree.setSelectionRow(0);
			updateMembranePanel( config.getSkinMembrane(),null,"Configuration: "+model.getSimulator().getCurrentConfig().getNumber()+". "+config.getSkinMembrane().toString(),true);
			

		}
		
		
		
	
		
	}
	

	
	
	private void updateMembranePanel(CellLikeMembrane membrane,CellLikeMembrane markedMembrane,String title,boolean paintMembrane)
	{
	
		if (paintMembrane)
		{
			panelMembrana.setMembrane(membrane);
			if (markedMembrane != membrane)
				panelMembrana.setMarkedMembrane(markedMembrane);
			else
				panelMembrana.setMarkedMembrane(null);
			panelMembrana.paint(panelMembrana.getGraphics());
		}
		
		
				
		this.setBorder(BorderFactory.createTitledBorder(BorderFactory
				.createLineBorder(Color.gray, 1), title, 
				TitledBorder.DEFAULT_JUSTIFICATION, 
				TitledBorder.DEFAULT_POSITION,
				null,
				null));
				//new Font("Dialog", Font.BOLD, 12), 
				//new Color(51, 51, 51)));
		
		
			
	}
	
	



	public void actionPerformed(ActionEvent e) {
	
	
		
		
			if ("Reset".equals(e
					.getActionCommand()))
			{
				model.getSimulator().reset();
			}
			else
			if ("Step Back".equals(e
					.getActionCommand()))
			{
				model.getSimulator().stepBack();
				
			}
			else
			if ("Step Forward".equals(e
						.getActionCommand()))
			{
				try
				{
				model.getSimulator().microStepExecution();
				model.getSimulator().microStepSelection();
				}
				catch(Exception ex){}
			}
			else
			if ("Run...".equals(e
							.getActionCommand()))
			{
				try
				{
					while(model.getSimulator().step());
				}
				catch(Exception ex){}
			}	
			model.notifyControllers();
			updateButtons();
	
	}
	
	private void updateButtons()
	{
		jButton.setEnabled(true);
		jButton1.setEnabled(true);
		jButton2.setEnabled(true);
		jButton3.setEnabled(true);
		
		if (model.getSimulator().isInitialConfig())
		{
			jButton.setEnabled(false);
			jButton1.setEnabled(false);
		}
		if (model.getSimulator().isFinished())
		{
			jButton2.setEnabled(false);
			jButton3.setEnabled(false);
		}
	}
	
	public void valueChanged(TreeSelectionEvent e) {
	    DefaultMutableTreeNode node = (DefaultMutableTreeNode)
	                       jTree.getLastSelectedPathComponent();
	    AmConfiguration config=(AmConfiguration)simulator.getCurrentConfig();
	    if (node == null)
	    {
	    	 updateMembranePanel(config.getSkinMembrane(),null,"Configuration 0.",true);
	    }
	    else
	    {
	        Object nodeInfo = node.getUserObject();
	         if (!(nodeInfo instanceof CellLikeMembrane))
	         {
	        	 updateMembranePanel(config.getSkinMembrane(),null,"Configuration 0.",true);
	        	 model.setSelectedMembrane(null);
	        	 objectsPanel.updateView();
	         }
	         else
	         {
	        	 model.setSelectedMembrane((CellLikeMembrane)nodeInfo);
	          	 updateMembranePanel(config.getSkinMembrane(),(CellLikeMembrane)nodeInfo,"Configuration: "+model.getSimulator().getCurrentConfig().getNumber()+". "+((CellLikeMembrane)nodeInfo).toString(),true);
	          	 objectsPanel.updateView();
	         }
	         
	    }
	     
	} 
	public void paint(Graphics g)
	{
		
		Dimension d = getSize();
		
		if (d.width>d.height+50)
			jSplitPane.setDividerLocation(d.width-d.height);
		
		super.paint(g);
	}

}
