/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.gui;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultMutableTreeNode;
import java.util.Collection;
import variants.am.AmPsystem;
import variants.am.rules.*;
import java.util.Iterator;
import java.util.ArrayList;
import util.Rule;




public class RuleTreeModel extends DefaultTreeModel{
	
	static final long serialVersionUID = 1;
	private Collection<Rule> evolutionRules=new ArrayList<Rule>();
	private Collection<Rule> sendInRules=new ArrayList<Rule>();
	private Collection<Rule> sendOutRules=new ArrayList<Rule>();
	private Collection<Rule> divisionRules=new ArrayList<Rule>();
	private Collection<Rule> disolutionRules=new ArrayList<Rule>();
	
	private AmPsystem psystem;
	public RuleTreeModel(AmPsystem psystem)
	{
		super(null,true);
		this.psystem=psystem;
		this.setRoot(makeRulesTree(0));
		reload();
		
	}
	
	private void updateCollections()
	{
		if (psystem==null)
			return;
		evolutionRules.clear();
		sendInRules.clear();
		sendOutRules.clear();
		divisionRules.clear();
		disolutionRules.clear();
		Iterator <Rule>it=psystem.getRules().iterator();
		while(it.hasNext())
		{
			Rule r = it.next();
			if (r instanceof AmEvolutionRule)
				evolutionRules.add(r);
			else
			if (r instanceof AmSendInRule)
				sendInRules.add(r);
			else
			if (r instanceof AmSendOutRule)
				sendOutRules.add(r);
			else
			if (r instanceof AmDivisionRule)
				divisionRules.add(r);
			else
				disolutionRules.add(r);
		}
	}
	
	private void addRulesToTreeNode(DefaultMutableTreeNode treeNode,Collection<Rule> rules)
	{
		Iterator <Rule>it = rules.iterator();
		while(it.hasNext())
			treeNode.add(new DefaultMutableTreeNode(it.next(),false));
	}
	
	private DefaultMutableTreeNode makeRulesTree(int order)
	{
		DefaultMutableTreeNode tree=new DefaultMutableTreeNode("Rules",true);
		
		switch(order)
		{
		/* order by rule type */
		case 0:
			updateCollections();
			DefaultMutableTreeNode evolutionTree= new DefaultMutableTreeNode("Evolution Rules",true);
			if (psystem!=null)
				addRulesToTreeNode(evolutionTree,evolutionRules);
			DefaultMutableTreeNode sendInTree= new DefaultMutableTreeNode("Send-In Rules",true);
			if (psystem!=null)
				addRulesToTreeNode(sendInTree,sendInRules);
			DefaultMutableTreeNode sendOutTree= new DefaultMutableTreeNode("Send-Out Rules",true);
			if (psystem!=null)
				addRulesToTreeNode(sendOutTree,sendOutRules);
			DefaultMutableTreeNode disolutionTree= new DefaultMutableTreeNode("Disolution Rules",true);
			if (psystem!=null)
				addRulesToTreeNode(disolutionTree,disolutionRules);
			DefaultMutableTreeNode divisionTree= new DefaultMutableTreeNode("Division Rules",true);
			if (psystem!=null)
				addRulesToTreeNode(divisionTree,divisionRules);
			tree.add(evolutionTree);
			tree.add(sendInTree);
			tree.add(sendOutTree);
			tree.add(disolutionTree);
			tree.add(divisionTree);
			break;
		case 1:
			break;
		case 2:
			break;
			
			
		
		}
		return tree;
	}
	public void updateTree(int view)
	{
		this.setRoot(makeRulesTree(view));
		
		reload();
	}

}
