/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.gui;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import java.awt.BorderLayout;
import javax.swing.BorderFactory;
import java.awt.Color;
import javax.swing.border.TitledBorder;

import javax.swing.JToolBar;
import javax.swing.JButton;
import javax.swing.JTree;

import java.awt.Component;


import variants.am.AmPsystem;




import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


public class RulesPanel extends JPanel implements ActionListener,AmGuiView{

		private static final long serialVersionUID = 1L;
		private JScrollPane jScrollPane = null;
		private JToolBar jToolBar = null;
		private JButton jButton = null;
		private JButton jButton1 = null;
		private JButton jButton2 = null;
		private JTree jTree = null;
		private RuleTreeModel ruleTreeModel=null;
		
		private AmGuiModel model;
		
		private Component owner;

		/**
		 * This is the default constructor
		 */
		public RulesPanel(AmGuiModel model,Component owner) {
			super();
			this.model=model;
			
			this.owner=owner;
			initialize();
		}

		
		public void updateView()
		{
			if (model.getSimulator().getPsystem().getRules().size()== 0) {
				jTree.setModel(new RuleTreeModel(null));
				ruleTreeModel = null;
				
			
			} else {
				if (ruleTreeModel == null) {
					ruleTreeModel = new RuleTreeModel((AmPsystem)model.getSimulator().getPsystem());
					jTree.setModel(ruleTreeModel);
				
				} else {
					ruleTreeModel.updateTree(0);
				}
				jTree.setSelectionRow(0);
				

			}
		}
		/**
		 * This method initializes this
		 * 
		 * @return void
		 */
		private void initialize() {
			this.setSize(300, 200);
			this.setLayout(new BorderLayout());
			this.setBorder(BorderFactory.createTitledBorder(BorderFactory
					.createLineBorder(Color.gray, 1),
					"Rules",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION,
					null, 
					null));
			this.add(getJScrollPane(), BorderLayout.CENTER);
			this.add(getJToolBar(), BorderLayout.SOUTH);
			updateView();
		
		}

		/**
		 * This method initializes jScrollPane	
		 * 	
		 * @return javax.swing.JScrollPane	
		 */
		private JScrollPane getJScrollPane() {
			if (jScrollPane == null) {
				jScrollPane = new JScrollPane();
				jScrollPane.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
				jScrollPane.setViewportView(getJTree());
			}
			return jScrollPane;
		}

		/**
		 * This method initializes jToolBar	
		 * 	
		 * @return javax.swing.JToolBar	
		 */
		private JToolBar getJToolBar() {
			if (jToolBar == null) {
				jToolBar = new JToolBar();
				jToolBar.setFloatable(false);
				//jToolBar.add(getJButton());
				//jToolBar.add(getJButton1());
				//jToolBar.add(getJButton2());
			}
			return jToolBar;
		}

		/**
		 * This method initializes jButton	
		 * 	
		 * @return javax.swing.JButton	
		 */
		private JButton getJButton() {
			if (jButton == null) {
				jButton = new JButton();
				jButton.setText("New");
				jButton.addActionListener(this);
			}
			return jButton;
		}

		/**
		 * This method initializes jButton1	
		 * 	
		 * @return javax.swing.JButton	
		 */
		private JButton getJButton1() {
			if (jButton1 == null) {
				jButton1 = new JButton();
				jButton1.setText("Delete");
			}
			return jButton1;
		}

		/**
		 * This method initializes jButton2	
		 * 	
		 * @return javax.swing.JButton	
		 */
		private JButton getJButton2() {
			if (jButton2 == null) {
				jButton2 = new JButton();
				jButton2.setText("Order by");
			}
			return jButton2;
		}

		/**
		 * This method initializes jTree	
		 * 	
		 * @return javax.swing.JTree	
		 */
		private JTree getJTree() {
			if (jTree == null) {
				jTree = new JTree(new RuleTreeModel(null));
				
			}
			return jTree;
		}
		
		
		
	
		
		public void actionPerformed(ActionEvent e)
		{
		   
		 
		}

	}

	

