/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.gui;


import java.awt.BorderLayout;
import java.awt.Graphics;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import java.awt.GridLayout;




public class SystemPanel extends JPanel {
	

	private AmGuiModel model=null;
	private static final long serialVersionUID = 1L;

	private JSplitPane jSplitPane1 = null;
	
	private MembranesPanel membranesPanel = null;
	private AlphabetPanel environmentPanel=null;
	private AlphabetPanel alphabetPanel = null;
	private AlphabetPanel objectsPanel=null;
	private RulesPanel rulesPanel = null;
	private JTabbedPane jTabbedPanel = null;
	private ConsolePanel consolePanel=null;
	public SystemPanel(AmGuiModel model) 
	{
		super();
		this.model = model;
		initialize();
		try
		{
			model.getSimulator().microStepSelection();
		}catch(Exception ex){}
	}
	

	
	




	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(640, 480);
		this.setLayout(new BorderLayout());
		this.add(getJSplitPane1(), BorderLayout.CENTER);
		
			
	}

	

	/**
	 * This method initializes jSplitPane1	
	 * 	
	 * @return javax.swing.JSplitPane	
	 */
	private JSplitPane getJSplitPane1() {
		if (jSplitPane1 == null) {
			jSplitPane1 = new JSplitPane();
			jSplitPane1.setRightComponent(getJTabbedPanel());
			jSplitPane1.setLeftComponent(getMembranesPanel());
			
		}
		return jSplitPane1;
	}

	

	/**
	 * This method initializes membranesPanel	
	 * 	
	 * @return plugins.activeMPlugin.guiPanels.MembranesPanel	
	 */
	private MembranesPanel getMembranesPanel() {
		if (membranesPanel == null) {
				membranesPanel = new MembranesPanel(model,this,getObjectsPanel());
				model.addController(new AmGuiController(membranesPanel));
		}
		return membranesPanel;
	}
	
	private JTabbedPane getJTabbedPanel()
	{
		if (jTabbedPanel==null)
		{
			jTabbedPanel= new JTabbedPane();
			jTabbedPanel.addTab("Rules",getRulesPanel());
			jTabbedPanel.addTab("Alphabet", getAlphabetPanel());
			jTabbedPanel.addTab("Console", getConsolePanel());
			jTabbedPanel.addTab("Membrane Objects",getObjectsPanel());
			jTabbedPanel.addTab("Environment",getEnvironmentPanel());
		}
		return jTabbedPanel;
	}

	public ConsolePanel getConsolePanel() {
		if (consolePanel == null) {
			consolePanel = new ConsolePanel(this);
			model.getSimulator().setInfoChannel(consolePanel.getPrintStream());
		}
		return consolePanel;
	}
	public AlphabetPanel getObjectsPanel() {
		if (objectsPanel == null) {
			objectsPanel = new AlphabetPanel(model,this,"Objects",AlphabetPanel.TYPE_OBJECTS);
			model.addController(new AmGuiController(objectsPanel));
		}
		return objectsPanel;
	}
	public AlphabetPanel getEnvironmentPanel() {
		if (environmentPanel == null) {
			environmentPanel = new AlphabetPanel(model,this,"Environment",AlphabetPanel.TYPE_ENVIRONMENT);
			model.addController(new AmGuiController(environmentPanel));
		}
		return environmentPanel;
	}

	/**
	 * This method initializes alphabetPanel	
	 * 	
	 * @return plugins.activeMPlugin.guiPanels.AlphabetPanel	
	 */
	private AlphabetPanel getAlphabetPanel() {
		if (alphabetPanel == null) {
			alphabetPanel = new AlphabetPanel(model,this,"Alphabet",AlphabetPanel.TYPE_ALPHABET);
		}
		return alphabetPanel;
	}

	/**
	 * This method initializes rulesPanel	
	 * 	
	 * @return plugins.activeMPlugin.guiPanels.RulesPanel	
	 */
	private RulesPanel getRulesPanel() {
		if (rulesPanel == null) {
			rulesPanel = new RulesPanel(model,this);
			model.addController(new AmGuiController(rulesPanel));
		}
		return rulesPanel;
	}

	
	
	
	public void paint(Graphics g)
	{
		
	
	
		jSplitPane1.setDividerLocation(0.8);
	
		super.paint(g);
	}
	
	
	
	
	
	
	
	
	
	
	

}  //  @jve:decl-index=0:visual-constraint="18,23"
