/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.rules;


import org.jdom.Attribute;
import org.jdom.Element;

import util.SemanticsException;
import util.CellLikeMembrane;
import util.PlinguaCoreException;
import util.Membrane;
import util.CellLikeMembrane;
import util.MultiSet;
public class AmDisolutionRule extends AmRule {
	
	private String rightHandObject;
	
	public AmDisolutionRule(String label, int charge,String leftHandObject,String rightHandObject)
	{
		super(label,charge,leftHandObject);
		if (rightHandObject==null)
			throw new NullPointerException("Null right-hand rule object");
		this.rightHandObject = rightHandObject;
	}

	public String getRightHandObject() {
		return rightHandObject;
	}

	
		
	public String toString() {
		
		return getRuleName()+": "+Membrane.getChargeSymbol(getCharge())+"["+getLeftHandObject()+"]'"+getLabel()+" --> "+getRightHandObject();
	}

	
	
	public void executeRule (CellLikeMembrane m,int applications,MultiSet<String>environment) throws PlinguaCoreException{
		
		if (applications>1)
			throw new PlinguaCoreException();
		
		CellLikeMembrane p = m.getParentMembrane();
		m.getMultiSet().remove(getLeftHandObject());
		try
		{
		m.disolution();
		}
		catch(SemanticsException ex)
		{
			throw new PlinguaCoreException(ex.getMessage());
		}
		p.getMultiSet().add(getRightHandObject());
		p.getNewMultiSet().add(getRightHandObject());
		
	}

	@Override
	public int getRuleType() {
		return DISOLUTION_RULE;
	}

	@Override
	public Element getElement() {
		Element e= super.getElement();
		Element e1=new Element("right_hand_rule");
		e1.setAttribute(new Attribute("object",rightHandObject));
		
		e.addContent(e1);
		return e;
	}
	
	
	
	

}
