/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.rules;
import org.jdom.Attribute;
import org.jdom.Element;

import util.Membrane;
import util.MultiSet;
import util.PlinguaCoreException;
import util.CellLikeMembrane;
import util.SemanticsException;

public class AmDivisionRule extends AmRule {
	
	private int newCharge1;
	private int newCharge2;
	private String rightHandObject1;
	private String rightHandObject2;
	
	public AmDivisionRule(String label,int charge,String leftHandObject,int newCharge1,String rightHandObject1,int newCharge2,String rightHandObject2) 
	{
		super(label,charge,leftHandObject);
	
		this.newCharge1 = newCharge1;
		this.rightHandObject1 = rightHandObject1;
		this.newCharge2 = newCharge2;
		this.rightHandObject2 = rightHandObject2;
	}
	
		
	public int getNewCharge1() {
		return newCharge1;
	}

	public int getNewCharge2() {
		return newCharge2;
	}

	public String getRightHandObject1() {
		return rightHandObject1;
	}
	
	public String getRightHandObject2() {
		return rightHandObject2;
	}
	
	
	
	public String toString() {
		
		return  getRuleName()+": "+Membrane.getChargeSymbol(getCharge())+"["+getLeftHandObject()+"]'"+getLabel()+" --> "+Membrane.getChargeSymbol(getNewCharge1())+"["+getRightHandObject1()+"] "+Membrane.getChargeSymbol(getNewCharge2())+"["+getRightHandObject2()+"]";
	}

	
	
	public void executeRule(CellLikeMembrane m,int applications,MultiSet<String>environment) throws PlinguaCoreException {
	
		if (applications>1)
			throw new PlinguaCoreException();
		m.getMultiSet().remove(getLeftHandObject());
		Membrane m2;
		try
		{
			m2=m.division();
		}
		catch(SemanticsException ex)
		{
			throw new PlinguaCoreException(ex.getMessage());
		}
		m.getMultiSet().add(getRightHandObject1());
		m.getNewMultiSet().add(getRightHandObject1());
		m.setCharge(getNewCharge1());
		m2.getMultiSet().add(getRightHandObject2());
		((CellLikeMembrane)m2).getNewMultiSet().addAll(m2.getMultiSet());
		m2.setCharge(getNewCharge2());
		
	}


	@Override
	public int getRuleType() {
		return DIVISION_RULE;
	}
	
	public Element getElement() {
		Element e= super.getElement();
		Element e1=new Element("right_hand_rule");
		e1.setAttribute(new Attribute("object",rightHandObject1));
		if (newCharge1>0)
			e1.setAttribute(new Attribute("charge","+1"));
		else
		if (newCharge1<0)
			e1.setAttribute(new Attribute("charge","-1"));
		else
			e1.setAttribute(new Attribute("charge","0"));
		e.addContent(e1);
	    e1=new Element("right_hand_rule");
		e1.setAttribute(new Attribute("object",rightHandObject2));
		if (newCharge2>0)
			e1.setAttribute(new Attribute("charge","+1"));
		else
		if (newCharge2<0)
			e1.setAttribute(new Attribute("charge","-1"));
		else
			e1.setAttribute(new Attribute("charge","0"));
		e.addContent(e1);
		return e;
	}
	

}
