/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.rules;
import org.jdom.Attribute;
import org.jdom.Element;

import util.MultiSet;
import util.HashMultiSet;
import util.Membrane;
import util.PlinguaCoreException;
import util.CellLikeMembrane;
import java.util.Iterator;

public class AmEvolutionRule extends AmRule{
	
	private MultiSet<String> rightHandObjects=null;
	
	public AmEvolutionRule(String label, int charge, String leftHandObject, MultiSet<String>rightHandObjects) 
	{
		super(label,charge,leftHandObject);
		this.rightHandObjects= new HashMultiSet<String>(rightHandObjects);
	}
	


	public MultiSet<String> getRightHandObjects() {
		return rightHandObjects;
	}
	

	public String toString() {
		
		return  getRuleName()+": "+Membrane.getChargeSymbol(getCharge())+"["+getLeftHandObject()+" --> "+rightHandObjects.toString()+"]'"+getLabel();
	}



	
	public void executeRule(CellLikeMembrane m,int applications,MultiSet<String>environment) throws PlinguaCoreException {
	
		if (applications>m.getMultiSet().count(getLeftHandObject()))
			throw new PlinguaCoreException();
		m.getMultiSet().remove(getLeftHandObject(),applications);
		m.getMultiSet().addAll(rightHandObjects,applications);
		m.getNewMultiSet().addAll(rightHandObjects,applications);
		
	}



	@Override
	public int getRuleType() {
		return EVOLUTION_RULE;
	}
	
	
	
	public Element getElement() {
		Element e= super.getElement();
		Iterator<String> it=rightHandObjects.entrySet().iterator();
		while(it.hasNext())
		{
			String obj=it.next();
			int mul=rightHandObjects.count(obj);
			Element e1= new Element("right_hand_rule");
			e1.setAttribute("object",obj);
			e1.setAttribute("multiplicity",String.valueOf(mul));
			e.addContent(e1);
		}
		return e;
	}
	
	

}
