/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.rules;

import org.jdom.Attribute;
import org.jdom.Element;

import util.Rule;
import util.Membrane;
import util.RuleReference;
import util.PlinguaCoreException;
import util.CellLikeMembrane;
import util.MultiSet;
public abstract class AmRule extends Rule{
	
	
	public static final int EVOLUTION_RULE=10;
	public static final int SEND_IN_RULE=20;
	public static final int SEND_OUT_RULE=30;
	public static final int DIVISION_RULE=40;
	public static final int DISOLUTION_RULE=50;
	private String leftHandObject;
		
	
	
	public AmRule(String label, int charge, String leftHandObject) 
	{
		super(label,charge);
		if (leftHandObject==null)
			throw new NullPointerException("Null left-hand rule object");
		this.leftHandObject=leftHandObject;
	}
	public String getRuleName()
	{
		switch(getRuleType())
		{
		case EVOLUTION_RULE:
			return "EVOLUTION RULE";
			
		case SEND_IN_RULE:
			return "SEND-IN RULE";
			
		case SEND_OUT_RULE:
			return "SEND-OUT RULE";
			
		case DIVISION_RULE:
			return "DIVISION RULE";
			
		case DISOLUTION_RULE:
			return "DISOLUTION RULE";
			
		default:
			return "";
		}
	}
		
	public Element getElement()
	{
		String name="";
		switch(getRuleType())
		{
		case EVOLUTION_RULE:
			name="evolution_rule";
			break;
		case SEND_IN_RULE:
			name="send_in_rule";
			break;
		case SEND_OUT_RULE:
			name="send_out_rule";
			break;
		case DIVISION_RULE:
			name="division_rule";
			break;
		case DISOLUTION_RULE:
			name="dissolution_rule";
			break;
		}
		Element e = new Element(name);
		e.setAttribute("label",getLabel());
		if (getCharge()>0)
			e.setAttribute(new Attribute("charge","+1"));
		else
		if (getCharge()<0)
			e.setAttribute(new Attribute("charge","-1"));
		else
			e.setAttribute(new Attribute("charge","0"));
		Element e1=new Element("left_hand_rule");
		e1.setAttribute(new Attribute("object",leftHandObject));
		e.setContent(e1);
		return e;
	}
	

	public String getLeftHandObject() {
		return leftHandObject;
	}


	public abstract int getRuleType();
	public abstract void executeRule(CellLikeMembrane m,int applications,MultiSet<String>environment) throws PlinguaCoreException;
	
	
}
