/*
 * This file is part of the pLinguaCore API 
 * pLinguaCore is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *  pLinguaCore is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with P-Lingua compiler.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  pLinguaCore 1.0 beta
 *  Copyright (c) Ignacio Perez-Hurtado 
 *  perezh@us.es
 *  Research Group On Natural Computing
 *  http://www.gcn.us.es
 */
package variants.am.rules;
import util.Membrane;
import util.PlinguaCoreException;
import util.CellLikeMembrane;
import util.MultiSet;
public class AmSendOutRule extends AmSendRule {
	
	public AmSendOutRule(String label, int charge,String leftHandObject,int newCharge,String rightHandObject)
	{
		super(label,charge,leftHandObject,newCharge,rightHandObject);
	}
	
	
	public String toString() {
		
		return  getRuleName()+": "+Membrane.getChargeSymbol(getCharge())+"["+getLeftHandObject()+"]'"+getLabel()+" --> "+Membrane.getChargeSymbol(getNewCharge())+"[]"+getRightHandObject();
	}
	

	@Override
	public void executeRule(CellLikeMembrane m,int applications,MultiSet<String>environment) throws PlinguaCoreException {
		
		if (applications>1)
			throw new PlinguaCoreException();
		
		m.getMultiSet().remove(getLeftHandObject());
		m.setCharge(getNewCharge());
		if (m.isSkinMembrane())
			environment.add(getRightHandObject());
		else
		{
			m.getParentMembrane().getMultiSet().add(getRightHandObject());
			m.getParentMembrane().getNewMultiSet().add(getRightHandObject());
		}
		
	}


	@Override
	public int getRuleType() {
		return SEND_OUT_RULE;
	}	
	

}
